import pandas as pd
import math
import numpy as np
from pathlib import Path
import streamlit as st
from taxontabletools2.utilities import collect_traits
from taxontabletools2.utilities import strip_traits
from taxontabletools2.utilities import collect_metadata
from taxontabletools2.utilities import load_df
from taxontabletools2.utilities import collect_replicates
from taxontabletools2.utilities import export_taxon_table
from taxontabletools2.utilities import simple_taxontable
from taxontabletools2.utilities import update_taxon_table

def presence_absence_table(taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df):
    ## create copies of the dataframes
    taxon_table_df = taxon_table_df.copy()
    metadata_df = metadata_df.copy()
    traits_df = traits_df.copy()

    ## set all element to 0/1
    taxon_table_df[samples] = taxon_table_df[samples].apply(lambda x: (x != 0).astype(int))

    ## export table
    suffix = 'PA'
    export_taxon_table(taxon_table_xlsx, taxon_table_df, traits_df, metadata_df, suffix)

def simplify_table(taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df):
    # merge duplicate OTUs
    save = False
    filtered_df = simple_taxontable(taxon_table_xlsx, taxon_table_df, samples, metadata_df, save)
    ## export the dataframe
    new_TaXon_table_xlsx = str(Path(taxon_table_xlsx)).replace('.xlsx', '_simplified.xlsx')
    with pd.ExcelWriter(new_TaXon_table_xlsx) as writer:
        filtered_df.to_excel(writer, sheet_name='Taxon Table', index=False)
        metadata_df.to_excel(writer, sheet_name='Metadata Table', index=False)

def add_traits_from_file(taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df, new_traits_df, taxon_col):
    ## create copies of the dataframes
    taxon_table_df = taxon_table_df.copy()
    metadata_df = metadata_df.copy()
    traits_df = traits_df.copy()

    for row in new_traits_df.values.tolist():
        taxon = row[0]
        traits = row[1:]
        for trait, trait_name in zip(traits, new_traits_df.columns.tolist()[1:]):
            # Find the index of the rows in taxon_table_df that match the taxon
            indices = taxon_table_df[taxon_table_df[taxon_col] == taxon].index
            # Add traits to each matching row in taxon_table_df
            for index in indices:
                traits_df.loc[index, trait_name] = trait

    traits_df = traits_df.fillna('')
    update_taxon_table(taxon_table_xlsx, taxon_table_df, traits_df, metadata_df, '')

def sort_samples(taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df):
    ## create copies of the dataframes
    taxon_table_df = taxon_table_df.copy()
    metadata_df = metadata_df.copy()
    traits_df = traits_df.copy()

    # Collect the sorted samples
    sorted_samples = metadata_df['Sample'].values.tolist()
    taxon_table_df_sorted = taxon_table_df[taxon_table_df.columns.tolist()[:9] + sorted_samples]

    update_taxon_table(taxon_table_xlsx, taxon_table_df_sorted, traits_df, metadata_df, '')

def rename_samples(taxon_table_xlsx, taxon_table_df, samples, metadata_df, traits_df, selected_metadata):
    # Create copies of the dataframes
    taxon_table_df = taxon_table_df.copy()
    metadata_df = metadata_df.copy()
    traits_df = traits_df.copy()

    # Collect the new sample names from the selected_metadata column
    sorted_samples = metadata_df['Sample'].values.tolist()
    new_names = metadata_df[selected_metadata].values.tolist()

    # Rename columns in taxon_table_df using the old and new sample names
    for old, new in zip(sorted_samples, new_names):
        taxon_table_df.rename(columns={old: new}, inplace=True)

    # Update the metadata DataFrame with the old and new sample names
    metadata_df['Old names'] = sorted_samples
    metadata_df['Sample'] = new_names

    # Call the update function to apply changes
    update_taxon_table(taxon_table_xlsx, taxon_table_df, traits_df, metadata_df, '')

def merge_ESV_tables(TaXon_table_xlsx, df1, df2, df1_metadata, df2_metadata, suffix):
    df1_traits = collect_traits(df1)
    df1_trait_names = df1_traits.columns.tolist()[1:]
    df1 = strip_traits(df1)
    df1_samples = df1.columns.tolist()[9:]
    df2_traits = collect_traits(df2)
    df2_trait_names = df2_traits.columns.tolist()[1:]
    df2 = strip_traits(df2)
    df2_samples = df2.columns.tolist()[9:]

    ## calculate shared traits
    shared_traits = list(set(df1_trait_names) & set(df2_trait_names))

    if len(set(df1_samples) & set(df2_samples)) != 0:
        st.error('Error: Cannot merge tables with overlapping sample names!')

    else:
        ## Collect some stats
        ESVs_1 = set(df1['ID'].values.tolist())
        ESVs_2 = set(df2['ID'].values.tolist())
        a_only = len(ESVs_1 - ESVs_2)
        shared = len(ESVs_1 & ESVs_2)
        b_only = len(ESVs_2 - ESVs_1)
        total = a_only + shared + b_only
        all_IDs = sorted(set(df1['ID'].values.tolist() + df2['ID'].values.tolist()))

        st.success(f'Shared ESVs: {shared}')
        st.success(f'Original exclusive ESVs: {a_only}')
        st.success(f'Added exclusive ESVs: {b_only}')
        st.success(f'Total ESVs: {total}')

        merged_taxon_table_df = pd.DataFrame()
        for ID in all_IDs:
            new_row = pd.DataFrame()

            # Case 1: Present in both datasets: Merge them and select the taxonomy with the higher similarity
            if ID in ESVs_1 and ID in ESVs_2:
                # data from table 1
                row1 = df1.loc[df1['ID'] == ID]
                data1 = row1.iloc[:, :9]
                similarity1 = row1['Similarity'].values.tolist()[0]
                reads1 = row1[df1_samples]
                traits1 = df1_traits.loc[df1_traits['ID'] == ID][shared_traits]
                traits1['ESV merge'] = ['Shared']

                # data from table 2
                row2 = df2.loc[df2['ID'] == ID]
                data2 = row2.iloc[:, :9]
                similarity2 = row2['Similarity'].values.tolist()[0]
                reads2 = row2[df2_samples]
                traits2 = df2_traits.loc[df2_traits['ID'] == ID][shared_traits]

                if data1.values.tolist() == data2.values.tolist():
                    # Find the index of the column "Seq"
                    col_index = data1.columns.get_loc("Seq")
                    # Insert the new DataFrame columns before "Seq"
                    data1_traits = pd.concat([data1.iloc[:, :col_index], traits1, data1.iloc[:, col_index:]], axis=1)
                    new_row = pd.concat([data1_traits.reset_index(drop=True),
                                         reads1.reset_index(drop=True),
                                         reads2.reset_index(drop=True)], axis=1)

                elif similarity1 > similarity2:
                    # Find the index of the column "Seq"
                    col_index = data1.columns.get_loc("Seq")
                    # Insert the new DataFrame columns before "Seq"
                    data1_traits = pd.concat([data1.iloc[:, :col_index], traits1, data1.iloc[:, col_index:]], axis=1)
                    new_row = pd.concat([data1_traits.reset_index(drop=True),
                                         reads1.reset_index(drop=True),
                                         reads2.reset_index(drop=True)], axis=1)
                else:
                    # Find the index of the column "Seq"
                    col_index = data2.columns.get_loc("Seq")
                    # Insert the new DataFrame columns before "Seq"
                    data2_traits = pd.concat([data2.iloc[:, :col_index], traits2, data2.iloc[:, col_index:]], axis=1)
                    new_row = pd.concat([data2_traits.reset_index(drop=True),
                                         reads1.reset_index(drop=True),
                                         reads2.reset_index(drop=True)], axis=1)

            # Case 2: Present in dataset 1: Fill up the other
            elif ID in ESVs_1:
                # data from table 1
                row1 = df1.loc[df1['ID'] == ID]
                data1 = row1.iloc[:, :9]
                reads1 = row1[df1_samples]
                traits1 = df1_traits.loc[df1_traits['ID'] == ID][shared_traits]
                traits1['ESV merge'] = ['Original']

                # data from table 2
                reads2 = pd.DataFrame([[0] * len(df2_samples)], columns=df2_samples)

                # Find the index of the column "Seq"
                col_index = data1.columns.get_loc("Seq")
                # Insert the new DataFrame columns before "Seq"
                data1_traits = pd.concat([data1.iloc[:, :col_index], traits1, data1.iloc[:, col_index:]], axis=1)
                new_row = pd.concat([data1_traits.reset_index(drop=True),
                                     reads1.reset_index(drop=True),
                                     reads2.reset_index(drop=True)], axis=1)

            # Case 2: Present in dataset 1: Fill up the other
            elif ID in ESVs_2:
                # data from table 1
                row2 = df2.loc[df2['ID'] == ID]
                data2 = row2.iloc[:, :9]
                reads2 = row2[df2_samples]
                traits2 = df2_traits.loc[df2_traits['ID'] == ID][shared_traits]
                traits2['ESV merge'] = ['Added']

                # data from table 2
                reads1 = pd.DataFrame([[0] * len(df1_samples)], columns=df1_samples)

                # Find the index of the column "Seq"
                col_index = data2.columns.get_loc("Seq")
                # Insert the new DataFrame columns before "Seq"
                data2_traits = pd.concat([data2.iloc[:, :col_index], traits2, data2.iloc[:, col_index:]], axis=1)
                new_row = pd.concat([data2_traits.reset_index(drop=True),
                                     reads1.reset_index(drop=True),
                                     reads2.reset_index(drop=True)], axis=1)

            else:
                print('Whoops that should not happen!')

            ## add to newly merged df
            merged_taxon_table_df = pd.concat([merged_taxon_table_df, new_row], ignore_index=True)

    ## the merged_df is the new Taxon Table
    ## now create the Metadata Table
    ## calculate shared metadata
    shared_metadata = list(set(df1_metadata.columns.tolist()) & set(df2_metadata.columns.tolist()))
    export_metadata_table_df = pd.concat([df1_metadata[shared_metadata], df2_metadata[shared_metadata]], ignore_index=True)
    export_metadata_table_df['ESV merge'] = ['Original'] * len(df1_metadata) + ['Added'] * len(df2_metadata)

    ## export df
    export_traits_df = collect_traits(merged_taxon_table_df)
    export_taxon_table_df = strip_traits(merged_taxon_table_df)
    export_taxon_table(TaXon_table_xlsx, export_taxon_table_df, export_traits_df, export_metadata_table_df, suffix)

    st.success(f'Tables were successfully merged!')



import streamlit as st
import pandas as pd
import numpy as np
import subprocess
import glob, sys, time, statistics, os.path, random
from pathlib import Path
import psutil
import plotly.graph_objects as go
from taxontabletools2.start import TTT_variables
from taxontabletools2.utilities import load_df
from taxontabletools2.utilities import collect_replicates
from taxontabletools2.utilities import collect_sample_stats
from taxontabletools2.basic_stats import basic_stats_reads
from taxontabletools2.basic_stats import basic_stats_OTUs
from taxontabletools2.basic_stats import taxonomic_resolution
from taxontabletools2.basic_stats import taxonomic_richness
from taxontabletools2.utilities import collect_traits
from taxontabletools2.utilities import strip_traits
from taxontabletools2.utilities import check_replicates
from taxontabletools2.table_processing import replicate_merging
from taxontabletools2.table_processing import negative_control_subtraction
from taxontabletools2.table_processing import read_based_filter
from taxontabletools2.table_processing import read_based_normalisation
from taxontabletools2.table_processing import taxonomic_filtering
from taxontabletools2.table_processing import sample_filtering
from taxontabletools2.taxontable_conversion import taxon_table_converter_ttt
from taxontabletools2.taxontable_conversion import taxon_table_converter_qiime2
import streamlit as st
from taxontabletools2.start import start

# Call the sidebar function
settings = start()
users_settings = settings['new_user_preferences_dict']

## Page title
st.write(""" ## API modules """)

# Ensure the session state variables are retained across subpages
if 'TaXon_table_df' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif 'TaXon_table_xlsx' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif st.session_state['TaXon_table_df'].empty == True:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please check your TaXon table. It seems to be empty!')

elif st.session_state['TaXon_table_xlsx'].is_file() == False:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### The TaXon table does not exist in the selected project folder!')

########################################################################################################################

else:
    st.write('### GBIF | Global Biodiversity Information Facility')

    with st.expander("See more"):

        gbif1, gbif2 = st.columns(2)
        taxids_dict = {'Animalia':'1',
                          'Archae':'2',
                          'Bacteria':'3',
                          'Chromista':'4',
                          'Fungi':'5',
                          'Plantae':'6',
                          'Protozoa':'7',
                          'Viruses':'8',
                          'CUSTOM':'2436436'
                          }
        with gbif1:
            selected_higher_taxon = st.selectbox('Select a higher taxon to filter:', ['Animalia',
                                                              'Archae',
                                                              'Bacteria',
                                                              'Chromista',
                                                              'Fungi',
                                                              'Plantae',
                                                              'Protozoa',
                                                              'Viruses',
                                                              'CUSTOM'
                                                              ])
            higherTaxonKey = taxids_dict[selected_higher_taxon]

        with gbif2:
            higherTaxonKey = st.number_input('GBIF higherTaxonKey:', value=int(higherTaxonKey))

        st.write('Collected information: species key, genus key, GBIF species, synonym, IUCN red list category, and habitat')
        st.write('Note: The collected traits are added to the active TaXon table.')

        if st.button('Fetch information from GBIF'):
            gbif_accession(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'], st.session_state['samples'], st.session_state['metadata_df'], st.session_state['traits_df'], higherTaxonKey)
            current_year = datetime.datetime.now().year
            current_date = datetime.datetime.now().strftime("%d %B %Y")
            st.success('Added accession numbers to species! Please remember to reload the TaXon table!')
            st.success(f"Citation: GBIF.org ({current_year}), GBIF Home Page. Available from: https://www.gbif.org [{current_date}]")








import streamlit as st
import pandas as pd
import numpy as np
import subprocess
import glob, sys, time, statistics, os.path, random
from pathlib import Path
import psutil
import plotly.graph_objects as go
from taxontabletools2.start import TTT_variables
from taxontabletools2.utilities import load_df
from taxontabletools2.utilities import collect_replicates
from taxontabletools2.utilities import collect_sample_stats
from taxontabletools2.basic_stats import basic_stats_reads
from taxontabletools2.basic_stats import basic_stats_OTUs
from taxontabletools2.basic_stats import taxonomic_resolution
from taxontabletools2.basic_stats import taxonomic_richness
from taxontabletools2.utilities import collect_traits
from taxontabletools2.utilities import strip_traits
from taxontabletools2.utilities import check_replicates
from taxontabletools2.table_processing import replicate_merging
from taxontabletools2.table_processing import negative_control_subtraction
from taxontabletools2.table_processing import read_based_filter
from taxontabletools2.table_processing import read_based_normalisation
from taxontabletools2.table_processing import taxonomic_filtering
from taxontabletools2.table_processing import sample_filtering
from taxontabletools2.taxontable_conversion import taxon_table_converter_ttt
from taxontabletools2.taxontable_conversion import taxon_table_converter_qiime2
from taxontabletools2.start import start

# Call the sidebar function
settings = start()
users_settings = settings['new_user_preferences_dict']

## Page title
st.write(""" ## Taxon Table conversion """)

a1, a2 = st.columns(2)
with a1:
    st.session_state['otu_table_path'] = st.file_uploader('Select your Read table:')
    st.session_state['otu_table_format'] = st.selectbox('Read table format', ['APSCALE'])

    if st.session_state['otu_table_path'] == None:
        st.write('Please select a Read table!')

with a2:
    st.session_state['taxonomy_table_path'] = st.file_uploader('Select your Taxonomy table:')
    st.session_state['taxonomy_table_format'] = st.selectbox('Taxonomy table format', ['APSCALE', 'BOLDigger'])
    st.session_state['genus_species_merge'] = st.selectbox('Merge the genus and species epithet column', ['Merge', 'No merge'], index=1)

    if st.session_state['taxonomy_table_path'] == None:
        st.write('Please select a Taxonomy table!')

## Save table to project folder
st.session_state['taxon_table_name'] = st.text_input('Taxon table name',
                                                     f'{st.session_state["path_to_outdirs"].name}_taxon_table') + '.xlsx'
st.session_state['taxon_table_path'] = st.session_state['path_to_outdirs'].joinpath('TaXon_tables',
                                                                                  st.session_state['taxon_table_name'])

if st.session_state['taxonomy_table_path'] != None and st.session_state['otu_table_path'] != None:
    if st.button('Convert to TaXon table format'):

        ## collect all individual colummns from the OTU table
        otu_table = pd.read_excel(st.session_state['otu_table_path']).fillna('')
        # otu_table = pd.read_excel('/Volumes/Coruscant/APSCALE_projects/GeDNA_MZB_all_apscale/7_otu_clustering/GeDNA_MZB_all_apscale_OTU_table.xlsx').fillna('')

        ## collect all individual columns from the taxonomy table
        if st.session_state['taxonomy_table_format'] == 'APSCALE':
            sheet_name = 'Taxonomy table'
        elif st.session_state['taxonomy_table_format'] == 'BOLDigger':
            sheet_name = 'BOLDigger hit'
        taxonomy_table = pd.read_excel(st.session_state['taxonomy_table_path'], sheet_name=sheet_name).fillna('')
        # taxonomy_table = pd.read_excel('/Volumes/Coruscant/APSCALE_projects/GeDNA_MZB_all_apscale/10_local_BLAST/BOLDResults_GeDNA_MZB_all_apscale_OTUs_part_1.xlsx', sheet_name=sheet_name).fillna('')

        ## check if the IDs are identical
        otu_table_IDs = otu_table['unique_ID'].values.tolist()
        taxonomy_table_IDs = taxonomy_table['unique_ID'].values.tolist()

        if otu_table_IDs != taxonomy_table_IDs:
            st.error('WARNING: The IDs of the Read table and Taxonomy table do not match! Make sure they are identically sorted!')
        else:
            ## merge the two dataframes
            taxon_table = taxonomy_table.merge(otu_table, how='inner', on='unique_ID')
            ## move the Seq column
            first_sample = otu_table.columns.tolist()[2]
            loc_first_sample = taxon_table.columns.tolist().index(first_sample)
            # Get the Seq column
            seq = taxon_table['Seq']
            # Drop the Seq column from its current position
            taxon_table.drop('Seq', axis=1, inplace=True)
            # Insert the Seq column at the desired location
            taxon_table.insert(loc_first_sample, 'Seq', seq)
            # Rename the ID column
            taxon_table.rename(columns={"unique_ID": "ID"}, inplace=True)

            ## create a metadata_df
            samples = otu_table.columns.tolist()[1:-1]
            metadata_df = pd.DataFrame([[i, ''] for i in samples], columns=['Sample', 'Metadata'])

            ## also add the column 'Kingdom' if that is not included
            if 'Kingdom' not in taxon_table.columns.tolist():
                # Get the index of the column 'Phylum'
                loc_phylum = taxon_table.columns.tolist().index('Phylum')
                # Insert the 'Kingdom' column at the desired location with value 'Life' for all rows
                taxon_table.insert(loc_phylum, 'Kingdom', 'Life')

            if st.session_state['genus_species_merge'] == 'Merge':
                taxon_table['Species'] = taxon_table.apply(
                    lambda row: row['Genus'] + ' ' + row['Species'] if row['Genus'] and row['Species'] else row[
                        'Species'], axis=1)

            ## Display the table!
            st.success('Successfully merged your tables:')
            st.dataframe(taxon_table)

            with pd.ExcelWriter(st.session_state['taxon_table_path']) as writer:
                taxon_table.to_excel(writer, sheet_name='Taxon Table', index=False)
                metadata_df.to_excel(writer, sheet_name='Metadata Table', index=False)
            st.success(f'Saved as \"{st.session_state["taxon_table_path"]}\"')


















import streamlit as st
import pandas as pd
import numpy as np
import subprocess
import glob, sys, time, statistics, os.path, random
from pathlib import Path
import psutil
import plotly.graph_objects as go
from taxontabletools2.start import TTT_variables
from taxontabletools2.utilities import load_df
from taxontabletools2.utilities import collect_replicates
from taxontabletools2.utilities import collect_sample_stats
from taxontabletools2.basic_stats import basic_stats_reads
from taxontabletools2.basic_stats import basic_stats_OTUs
from taxontabletools2.basic_stats import taxonomic_resolution
from taxontabletools2.basic_stats import taxonomic_richness
from taxontabletools2.utilities import collect_traits
from taxontabletools2.utilities import strip_traits
from taxontabletools2.utilities import check_replicates
from taxontabletools2.table_processing import replicate_merging
from taxontabletools2.table_processing import negative_control_subtraction
from taxontabletools2.table_processing import read_based_filter
from taxontabletools2.table_processing import read_based_normalisation
from taxontabletools2.table_processing import taxonomic_filtering
from taxontabletools2.table_processing import sample_filtering
from taxontabletools2.taxontable_conversion import taxon_table_converter_ttt
from taxontabletools2.taxontable_conversion import taxon_table_converter_qiime2
from taxontabletools2.time_series import time_series_richness_with_ci
import streamlit as st
from taxontabletools2.start import start

import numpy as np
import pymannkendall as mk

# Data generation for analysis
data = np.random.rand(360,1)

result = mk.original_test(data)
print(result)

# Call the sidebar function
settings = start()
users_settings = settings['new_user_preferences_dict']

## Page title
st.write(""" ## Time series """)

# Ensure the session state variables are retained across subpages
if 'TaXon_table_df' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif 'TaXon_table_xlsx' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif st.session_state['TaXon_table_df'].empty == True:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please check your TaXon table. It seems to be empty!')

elif st.session_state['TaXon_table_xlsx'].is_file() == False:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### The TaXon table does not exist in the selected project folder!')

########################################################################################################################

else:
    st.write('### Richness')

    time1, time2, time3, time4, time5 = st.columns(5)

    with time1:
        available_metadata = ['All samples'] + st.session_state['metadata_df'].columns.tolist()[1:]
        selected_metadata = st.selectbox('Select metadata to test:', available_metadata)

    with time2:
        taxonomic_level_1 = st.selectbox('Select first taxonomic level:', st.session_state['available_taxonomic_levels_list'], index=5)

    with time3:
        taxonomic_level_2 = st.selectbox('Select second taxonomic level:', st.session_state['available_taxonomic_levels_list'], index=3)

    with time4:
        n_bootstraps = st.number_input('Select number of LOESS bootstraps:', min_value=0, max_value=10000, value=1000)

    with time5:
        ci_level = st.number_input('Select confidence intervall :', min_value=0, max_value=100, value=95)

    if st.button(f'Calculate {taxonomic_level_1} venn diagram for {selected_metadata}'):

        # Call the venn function and render it within Streamlit
        time_series_richness_with_ci_fig = time_series_richness_with_ci(
            st.session_state['path_to_outdirs'],
            st.session_state['TaXon_table_xlsx'],
            st.session_state['TaXon_table_df'],
            st.session_state['samples'],
            st.session_state['metadata_df'],
            selected_metadata,
            st.session_state['traits_df'],
            '',
            users_settings,
            {'selected_metadata': selected_metadata,
             'taxonomic_level_1': taxonomic_level_1,
             'taxonomic_level_2': taxonomic_level_2,
             'n_bootstraps': n_bootstraps,
             'ci_level': ci_level
             }
        )

        st.plotly_chart(time_series_richness_with_ci_fig, use_container_width=True, config=st.session_state['config'])


import subprocess
import sys
import os
from pathlib import Path
import pandas as pd
import tkinter as tk
from tkinter import filedialog, messagebox, simpledialog
from update_checker import update_check
import importlib.metadata
import threading

## Check for updates
def check_package_update(package_name):
    try:
        # Get the currently installed version
        installed_version = importlib.metadata.version(package_name)
    except importlib.metadata.PackageNotFoundError:
        return f"{package_name} is not installed."

    # Check for updates
    res = update_check(package_name, installed_version)
    if res:
        return f"New version of {package_name} is available: {res['latest_version']} (Installed: {installed_version})"
    else:
        return f"{package_name} is up to date (Version: {installed_version})."

# Function to validate the output path
def validate_output_path(path):
    """Check if the given path exists and is a directory."""
    return os.path.isdir(path)

# Function to select a folder
def select_folder():
    global path_to_outdirs
    new_folder = filedialog.askdirectory(title="Select the projects folder")
    if new_folder and validate_output_path(new_folder):
        path_to_outdirs = new_folder
        folder_label.config(text=f"Selected Folder: {path_to_outdirs}")
    else:
        messagebox.showerror("Invalid Folder", "Please select a valid folder.")
        folder_label.config(text=f"Using previous folder: {path_to_outdirs}")

# Function to create a Default_project folder if no projects exist
def ensure_default_project(folder):
    """Check if the folder contains subdirectories (projects) and create Default_project if none exist."""
    sub_folders = [
                    "Alpha_diversity",
                    "Basic_stats",
                    "Beta_diversity",
                    "CCA_plots",
                    "GBIF",
                    "Krona_charts",
                    "Meta_data_table",
                    "NMDS_plots",
                    "Occurrence_analysis",
                    "ParCat_plots",
                    "PCoA_plots",
                    "Per_taxon_statistics",
                    "Perlodes",
                    "Rarefaction_curves",
                    "RAW_tables",
                    "Read_proportions_plots",
                    "Replicate_analysis",
                    "Site_occupancy_plots",
                    "Table_comparison",
                    "Taxon_lists",
                    "TaXon_tables",
                    "TaXon_tables_per_sample",
                    "Taxonomic_resolution_plots",
                    "Taxonomic_richness_plots",
                    "Venn_diagrams"
                ]

    folder_path = Path(folder)
    # Check if the folder is empty or contains only hidden files
    if not any(item.is_dir() or item.is_file() for item in folder_path.iterdir() if not item.name.startswith('.')):
        default_project_path = folder_path.joinpath("Default_project")
        default_project_path.mkdir(parents=True, exist_ok=True)
        # Create subdirectories inside the Default_project folder
        for sub_folder in sub_folders:
            sub_folder_path = default_project_path.joinpath(sub_folder)
            sub_folder_path.mkdir(parents=True, exist_ok=True)
        print(f"Default_project folder created at: {default_project_path}")
    else:
        print(f"Projects found in the selected folder. Default_project folder is not needed.")

# Function to create a new project folder
def create_new_project():
    if not path_to_outdirs or not validate_output_path(path_to_outdirs):
        messagebox.showwarning("Warning", "Please select a valid projects folder first.")
        return

    project_name = simpledialog.askstring("Create New Project", "Enter the name of the new project:")
    if not project_name or not project_name.strip():
        messagebox.showerror("Invalid Name", "Project name cannot be empty.")
        return

    project_path = Path(path_to_outdirs).joinpath(project_name.strip())
    if project_path.exists():
        messagebox.showerror("Error", f"A project named '{project_name}' already exists.")
    else:
        project_path.mkdir(parents=True, exist_ok=True)
        sub_folders = [
            "Alpha_diversity",
            "Basic_stats",
            "Beta_diversity",
            "CCA_plots",
            "GBIF",
            "Krona_charts",
            "Meta_data_table",
            "NMDS_plots",
            "Occurrence_analysis",
            "ParCat_plots",
            "PCoA_plots",
            "Per_taxon_statistics",
            "Perlodes",
            "Rarefaction_curves",
            "RAW_tables",
            "Read_proportions_plots",
            "Replicate_analysis",
            "Site_occupancy_plots",
            "Table_comparison",
            "Taxon_lists",
            "TaXon_tables",
            "TaXon_tables_per_sample",
            "Taxonomic_resolution_plots",
            "Taxonomic_richness_plots",
            "Venn_diagrams"
        ]
        for sub_folder in sub_folders:
            project_path.joinpath(sub_folder).mkdir(parents=True, exist_ok=True)
        messagebox.showinfo("Success", f"New project '{project_name}' created successfully.")

# Function to run the Streamlit app
def run_streamlit_app():
    try:
        # Get the directory of the current script (__main__.py)
        script_dir = os.path.dirname(os.path.abspath(__file__))

        # Construct the path to taxontabletools_2.0.py relative to __main__.py
        script_path = os.path.join(script_dir, 'taxontabletools_2.0.py')

        print('Press CTRL + C to close APSCALE!')
        subprocess.call(['streamlit', 'run', script_path, '--theme.base', 'light', '--server.address', 'localhost'])
    except KeyboardInterrupt:
        sys.exit()

# Function to save the selected folder to the preferences file
def save_folder_to_preferences():
    user_preferences_df.loc[user_preferences_df['Variable'] == 'path_to_outdirs', 'Value'] = path_to_outdirs
    user_preferences_df.to_excel(user_preferences_xlsx, index=False)

# Function to close the tkinter app and continue with the script
def start_app():
    global path_to_outdirs

    # Validate the current output path
    if not path_to_outdirs or not validate_output_path(path_to_outdirs):
        messagebox.showwarning("Warning", "Please select a valid projects folder before continuing.")
        return

    # Save the selected folder to user_preferences.xlsx
    save_folder_to_preferences()

    # Ensure a Default_project exists if no other projects are present
    ensure_default_project(path_to_outdirs)

    # Hide window
    root.withdraw()

    # Run the Streamlit app in a new thread to avoid freezing the GUI
    threading.Thread(target=run_streamlit_app).start()

# Tkinter window setup
root = tk.Tk()
root.title("TaxonTableTools Start Window")
root.geometry("400x300")

# Check for APSCALE package update
update_info = check_package_update('taxontabletools')

# Update label
update_label = tk.Label(root, text=update_info, wraplength=300, justify="left")
update_label.pack(pady=10)

# Paths and user preferences setup
path_to_ttt = Path(__file__).resolve().parent
user_preferences_xlsx = path_to_ttt.joinpath('user_preferences.xlsx')

# Read user preferences from the file
user_preferences_df = pd.read_excel(user_preferences_xlsx).fillna('')
path_to_outdirs = user_preferences_df.loc[user_preferences_df['Variable'] == 'path_to_outdirs', 'Value'].values[0]

# Validate the initial path
if not validate_output_path(path_to_outdirs):
    path_to_outdirs = ''

# Display the current folder
folder_label = tk.Label(root, text=f"Current folder: {path_to_outdirs if path_to_outdirs else 'No folder selected'}")
folder_label.pack(pady=10)

# Button to select a new folder
select_button = tk.Button(root, text="Select Projects Folder", command=select_folder)
select_button.pack(pady=10)

# Button to create a new project
new_project_button = tk.Button(root, text="Create New Project", command=create_new_project)
new_project_button.pack(pady=10)

# Start button to continue with the script
start_button = tk.Button(root, text="Start", command=start_app)
start_button.pack(pady=10)

# Run tkinter main loop
root.mainloop()


import streamlit as st
import pandas as pd
import numpy as np
import subprocess
import glob, sys, time, statistics, os.path, random
from pathlib import Path
import psutil
import plotly.graph_objects as go
from taxontabletools2.start import TTT_variables
from taxontabletools2.utilities import load_df
from taxontabletools2.utilities import collect_replicates
from taxontabletools2.utilities import collect_sample_stats
from taxontabletools2.basic_stats import basic_stats_reads
from taxontabletools2.basic_stats import basic_stats_OTUs
from taxontabletools2.basic_stats import taxonomic_resolution
from taxontabletools2.basic_stats import taxonomic_richness
from taxontabletools2.utilities import collect_traits
from taxontabletools2.utilities import strip_traits
from taxontabletools2.utilities import check_replicates
from taxontabletools2.table_processing import replicate_merging
from taxontabletools2.table_processing import negative_control_subtraction
from taxontabletools2.table_processing import read_based_filter
from taxontabletools2.table_processing import read_based_normalisation
from taxontabletools2.table_processing import taxonomic_filtering
from taxontabletools2.table_processing import sample_filtering
from taxontabletools2.taxontable_conversion import taxon_table_converter_ttt
from taxontabletools2.taxontable_conversion import taxon_table_converter_qiime2
import streamlit as st
from taxontabletools2.start import start

# Call the sidebar function
settings = start()
users_settings = settings['new_user_preferences_dict']

## Page title
st.write(""" ## Table processing """)

# Ensure the session state variables are retained across subpages
if 'TaXon_table_df' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif 'TaXon_table_xlsx' not in st.session_state:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please load a TaXon table to continue!')

elif st.session_state['TaXon_table_df'].empty == True:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### Please check your TaXon table. It seems to be empty!')

elif st.session_state['TaXon_table_xlsx'].is_file() == False:
    error1, error2, error3 = st.columns(3)
    with error2:
        st.write('##### The TaXon table does not exist in the selected project folder!')

########################################################################################################################

else:
    st.write('### 1) Replicate merging ')

    with st.expander("See more"):
        replicate_dict = collect_replicates(st.session_state['samples'])

        n_samples = len(replicate_dict[0])
        n_replicates_suffixes = len(replicate_dict[1])
        a1, a2 = st.columns(2)
        with a1:
            replicate_selector = st.selectbox('Replicates select:', ['Manual', 'Detect from file'])

            if replicate_selector == 'Detect from file':
                st.write(f'Replicate suffixes: {replicate_dict[1]}')
            else:
                replicates_suffixes = st.text_input('Enter replicates suffixes (separate by comma):', 'A,B')
                replicates_suffixes = [i for i in replicates_suffixes.split(',')]
                st.write(f'Replicate suffixes: {replicates_suffixes}')
                n_replicates_suffixes = len(replicates_suffixes)
                replicate_dict[1] = replicates_suffixes

        with a2:
                replicate_cutoff = st.selectbox('Replicates cutoff:', [i for i in range(1, n_replicates_suffixes + 1)][::-1])
                st.write(f'OTU must be present in {replicate_cutoff} of {n_replicates_suffixes} replicates!')

                replicate_non_matching_samples = st.selectbox('Samples without matching replicates:', ['Keep (without merging)', 'Remove from dataset'])

        test, missing_samples = check_replicates(replicate_dict, st.session_state['samples'])
        if test == False:
            st.error('Warning: Not all samples are present with all expected replicates. Please check you table!')
            st.error(', '.join(list(missing_samples)[:20]))

        if st.button('Merge replicates'):
            replicate_merging(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'], st.session_state['samples'], st.session_state['metadata_df'], st.session_state['traits_df'], replicate_cutoff, replicate_dict, replicate_non_matching_samples)
            st.success('Merged replicates. Remember to reload the TaXon table!')



    ########################################################################################################################
    st.write('### 2) Negative control subtraction ')
    with st.expander("See more"):
        b1, b2 = st.columns(2)
        with b1:
            search_mask = st.text_input('Enter negative control identifier:', 'NC_')
        with b2:
            mode = st.selectbox('Select how to deal with negatice control subtraction', ['Sum of all NCs', 'Maximum in all NCs', 'Average in all NCs',])
        samples_to_remove = st.multiselect('Selected NCs:', [i for i in st.session_state['samples'] if search_mask in i], default=[i for i in st.session_state['samples'] if search_mask in i])
        if st.button('Subtract controls'):
            negative_control_subtraction(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'], st.session_state['samples'], st.session_state['metadata_df'], st.session_state['traits_df'], mode, samples_to_remove)
            st.success(f'Subtracted reads from {len(samples_to_remove)} negative controls. Remember to reload the TaXon table!')

    ########################################################################################################################
    st.write('### 3) Read-based filter ')
    with st.expander("See more"):
        c1, c2 = st.columns(2)
        with c1:
            read_filter_mode = st.selectbox('Read-based filter mode:', ['Relative', 'Absolute'])
            if read_filter_mode == 'Absolute':
                read_filter_threshold = st.number_input('Cutoff:', min_value=0, value=1000)
            else:
                read_filter_threshold = st.number_input('Cutoff (%):', min_value=0.000, max_value=100.0, value=0.01)
            if st.button('Read-based filter'):
                read_based_filter(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'], st.session_state['samples'], st.session_state['metadata_df'], st.session_state['traits_df'], read_filter_mode, read_filter_threshold)
                st.success(f'Filtered samples according to reads. Remember to reload the TaXon table!')
        with c2:
            read_normalization = st.selectbox('Normalization filter mode:', ['Average', 'Minimum', 'Custom'])
            if read_normalization == 'Average':
                n_reads = round(st.session_state['TaXon_table_df'][st.session_state['samples']].sum().mean())
                normalization_threshold = st.number_input('Normalization read threshold:', min_value=0, value=n_reads)
            elif read_normalization == 'Minimum':
                n_reads = min(st.session_state['TaXon_table_df'][st.session_state['samples']].sum())
                normalization_threshold = st.number_input('Normalization read threshold:', min_value=0, value=n_reads)
            else:
                normalization_threshold = st.number_input('Normalization read threshold:', min_value=0,
                                                          value=60000)
            if st.button('Read-based normalization'):
                read_based_normalisation(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'], st.session_state['samples'], st.session_state['metadata_df'], st.session_state['traits_df'], normalization_threshold)
                st.success(f'Normalized samples according to reads. Remember to reload the TaXon table!')


    ########################################################################################################################
    st.write('### 4) Taxonomic filtering ')
    with st.expander("See more"):
        d1, d2 = st.columns(2)
        with d1:
            taxonomic_level = st.selectbox('Select a taxonomic level:', st.session_state['available_taxonomic_levels_list'])
            available_taxa = sorted(set(st.session_state['TaXon_table_df'][taxonomic_level].values.tolist()))
        with d2:
            selector = st.selectbox('Select mode:', ['Select all available taxa', 'Select no taxa'])
        if selector == 'Select all available taxa':
            taxa_to_remove = st.multiselect(' ', available_taxa, default=available_taxa)
        else:
            taxa_to_remove = st.multiselect(' ', available_taxa)
        taxonomic_filtering_suffix = st.text_input('Enter suffix for the filtered table:', '', key='taxonomic_filtering_suffix')
        if st.button('Remove selected taxa'):
            mode= 'Remove'
            taxonomic_filtering(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'],
                                st.session_state['samples'], st.session_state['metadata_df'],
                                st.session_state['traits_df'], mode, taxa_to_remove, taxonomic_level, taxonomic_filtering_suffix)
            st.success(f'Removed {len(taxa_to_remove)} taxa. Remember to reload the TaXon table!')
        if st.button('Keep selected taxa'):
            mode = 'Keep'
            taxonomic_filtering(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'],
                                st.session_state['samples'], st.session_state['metadata_df'],
                                st.session_state['traits_df'], mode, taxa_to_remove, taxonomic_level, taxonomic_filtering_suffix)
            st.success(f'Kept {len(taxa_to_remove)} taxa. Remember to reload the TaXon table!')

    ########################################################################################################################
    st.write('### 5) Sample filtering ')
    with st.expander("See more"):
        e1, e2 = st.columns(2)
        with e1:
            search_mask = st.text_input('Type to filter for samples:', '', key='sample_filtering_search_mask')
        with e2:
            selector = st.selectbox('Select mode:', ['Select all available samples', 'Select no samples'])
        samples_to_remove = st.multiselect('Selected samples:', [i for i in st.session_state['samples'] if search_mask in i], default=[i for i in st.session_state['samples'] if search_mask in i])
        sample_filtering_suffix = st.text_input('Enter suffix for the filtered table:', '', key='sample_filtering_suffix')
        if st.button('Remove selected samples'):
            mode = 'Remove'
            sample_filtering(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'],
                                st.session_state['samples'], st.session_state['metadata_df'],
                                st.session_state['traits_df'], mode, samples_to_remove, sample_filtering_suffix)
            st.success(f'Removed {len(samples_to_remove)} samples. Remember to reload the TaXon table!')
        if st.button('Keep selected samples'):
            mode = 'Keep'
            sample_filtering(st.session_state['TaXon_table_xlsx'], st.session_state['TaXon_table_df'],
                                st.session_state['samples'], st.session_state['metadata_df'],
                                st.session_state['traits_df'], mode, samples_to_remove, sample_filtering_suffix)
            st.success(f'Kept {len(samples_to_remove)} samples. Remember to reload the TaXon table!')